




--// TSLibrary_Misc.e
--// Tone koda <tone.skoda@siol.net>
--// Created on 30. September 2001.
--// Miscalenous general stuff.





--//--//--//--//--//--//--//--//--//-- Variables, constants ... --//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//

constant false  = 0
constant true   = 1





--//--//--//--//--//--//--//--//--//-- Routines. --//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//--//


--/*
-- file_exists [Created on 30. September 2001, 11:48]
-- The 'file_exists' function tests if file exists.
--
-- PARAMETERS
-- 'file_name'
--    Name of the file.
--
-- RETURN VALUES
-- If file exists, true.
--
-- If file doesn't exist, false.
--
--*/
global function file_exists (sequence file_name)
	integer a
	a = open (file_name, "r")
	if a = -1 then
		return false
	else
		close(a)
		return true
	end if	
end function

-- returns file name
-- example: "C:\dir\file.txt" returns "file.txt"
global function get_file_name (sequence fname)
	for i = length (fname) to 1 by -1 do
		if fname [i] = '\\' or fname [i] = '/' then
			return fname [i + 1 .. length (fname)]
		end if
	end for
	return fname
end function

-- returns file title
-- example: "C:\dir\file.txt" returns "file"
global function get_file_title (sequence fname)
	integer c -- current char
	integer dot_pos, i
	-- find last dot position
	i = length (fname)
	while 1 do
		if i < 1 then
			dot_pos = length (fname) + 1
			exit
		end if
		c = fname [i]
		if c = '.' then
			dot_pos = i
			exit
		elsif c = '\\' or c = '/' then
			dot_pos = length (fname) + 1
			exit
		end if
		i -= 1
	end while
	--find last slash position
	for j = dot_pos - 1 to 1 by -1 do
		c = fname [j]
		if c = '\\' or c = '/' then
			return fname [j + 1 .. dot_pos - 1]
		end if
	end for
	return fname [1 .. dot_pos - 1]
end function

-- returns file extension
-- example: "C:\dir\file.txt" returns "txt"
global function get_file_ext (sequence fname)
	for i = length (fname) to 1 by -1 do
		if fname [i] = '.' then
			return fname [i + 1 .. length (fname)]
		elsif fname [i] = '\\' or fname [i] = '/' then
			return ""
		end if
	end for
	return ""
end function

-- returns file directory
-- example: "C:\dir\file.txt" returns "C:\dir"
global function get_file_directory (sequence fname)
	integer c -- current char
	for i = length (fname) to 1 by -1 do
		c = fname [i]
		if c = '\\' or c = '/' then
			return fname [1 .. i - 1]
		elsif c = '.' then
			for j = i - 1 to 1 by -1 do
				c = fname [j]
				if c = '\\' or c = '/' then
					return fname [1 .. j - 1]
				end if
			end for
			return ""
		end if
	end for
	return fname
end function

--/*
-- remove_member [Created on 4. October 2001, 16:36]
-- The 'remove_member' function removes a member with index 'index' from sequence 's'.
--
-- PARAMETERS
-- 'index'
--    Index of the member to be removed.
--
-- 's'
--    Sequence from which to remove one member.
--
-- RETURN VALUES
-- Sequence 's' with one element removed.
--
-- REMRAKS
-- If index is out of range, the original sequence is returned. This functiion never fails.
--
--*/
global function remove_member (integer index, sequence s)
    if index < 1 or index > length (s) then --// Index is out of range.
        return s
    end if
    if index = length (s) then  --// Trying to remove last member from sequence.
        return s  [1 .. length (s) - 1]
    else                        --// Not trying to remove last member from sequence.
        return s  [1 .. index - 1] & s  [index + 1 .. length (s)]
    end if
end function

--// finds string in sequence with strings,  case is irrelavant
--// returns index where in s found or 0 if not found
global function findstr_nocase (sequence s, sequence str, integer from)
    sequence str_lower
    str_lower = lower (str)
    for i = from to length (s) do
	    if equal (str_lower, lower (s [i])) then
	        return i
	    end if
    end for
    return 0
end function

--/*
-- ensure_short_cut_front [Created on 7. October 2001, 01:35]
-- The 'ensure_short_cut_front' function ensures that sequence 's' is not longer that 'how_short'.
-- If it is longer than some excessive members are cut from the front.
-- PARAMETERS
-- 's'
--    Sequence to be checked for length and cut if necessary.
-- 'how_short'
--    How short should sequence be. If it is same long or shorter than this number it's ok.
--
-- RETURN VALUES
-- Cut or unchanged sequence.
--
--*/
global function ensure_short_cut_front (sequence s, integer how_short)
    if length (s) > how_short then --// Sequence is too big.
        return s[length (s) - how_short + 1 .. length (s)]
    end if
    return s
end function