-- tk_abort.e -- Error Message Subsystem
-- Last Changed on 1/05/2001 at 6:43:30 by Derek Parnell --
include msgbox.e
-----------------------------------------------------------------------------
integer
    vShowWarnings        -- if false, suppresses warning messages

object
    vUserCleanUp,        -- the user's "clean up" routine(s)
    vUserCleanUpData
    

sequence
     vMsgsToIgnore,
     vVersion,
     vAppName,
     vCleanUp_parms
        
vMsgsToIgnore = {}
vUserCleanUp = -1       -- defined by user
vUserCleanUpData = -1
vShowWarnings = 1     -- display warning messages

------------------------------------------------
procedure fCleanUp(integer pErrCode, sequence pDispMsg)
------------------------------------------------
integer lRC, lNumUserIds
object lUserCleanUp, lUserCleanUpData
   -- user clean up
   if sequence(vUserCleanUp)
   then
      lUserCleanUp = vUserCleanUp
      lUserCleanUpData = vUserCleanUpData
      vUserCleanUp = -1
      lNumUserIds = length(lUserCleanUp)
      for i = 1 to length(lUserCleanUp) do
         lRC = call_func( lUserCleanUp[i], 
                           {pErrCode, pDispMsg, (i = lNumUserIds),
                            lUserCleanUpData[i]} )
         if lRC = -1
         then
            exit
         end if
      end for
      vUserCleanUp = lUserCleanUp
   end if    
    
end procedure

procedure fShowError( sequence pParams, integer pMode )
integer lErrCode,lResponse,lStyle

sequence s,lDispMsg,lTitle


   if length(pParams) >= 2
         and
      sequence(pParams[1])
   then
      s = pParams[1]
      if integer(pParams[2])
      then
         lErrCode = pParams[2]
      else
         lErrCode = 0
      end if
      if length(pParams) >= 3
      then
         s = sprintf(s, pParams[3])
      end if
      lDispMsg = sprintf("Error code %d\n%s", {lErrCode, s})
   else
      s = pParams
      lErrCode = 0
      lDispMsg = s
   end if
                  
    lTitle = vAppName & " - "
   if pMode = 1
   then
      lTitle &= "Error Warning"
      lDispMsg &= "\n\n" &
                  "Press\n" &
                  "  [ YES ]  to continue,\n" &
                  "  [ NO ] to ignore repeats, or\n" &
                  "  [ Cancel ]  to quit."
      lStyle = MB_ICONEXCLAMATION + MB_YESNOCANCEL
   else
      lTitle &= "Fatal Error"
      lStyle = MB_ICONHAND
   end if

   -- give message
   if find(lErrCode, vMsgsToIgnore) = 0
   then
      lDispMsg &= "\n\n" & vVersion
      lResponse = message_box( lDispMsg, lTitle, lStyle+MB_TASKMODAL )
   else
      lResponse = IDYES
   end if

   if pMode = 2
         or
      lResponse = IDCANCEL
   then
      fCleanUp(lErrCode, lDispMsg)

      if pMode = 2 or lErrCode = 999
      then
          -- abort
         abort(1/0) -- Force the ex.err to be created.
      else
         abort(0) -- Quit silently.
      end if
   end if

   if lResponse = IDNO
   then
      vMsgsToIgnore &= lErrCode
   end if
   
end procedure



-----------------------------------------------------------------------------
--/topic Utilities
--/proc abortErr( sequence errorparam )
--/desc Display an error message, run any user-defined cleanup routines, and then abort.
-- /i errorparam is a one, two or three element sequence. /n
-- The first element is a text message. This is mandatory. /n
-- The second element is an error code (integer). The default is zero. /n
-- The third elements is a list of values for the "sprint" pattern in the
-- first element (a sequence).
--
-- If any user-defined clean up routine have been attached, they are invoked prior to
-- win32lib's own cleanup routine, then the application is aborted. See /attachCleanUp()
-- for details.
--
-- Example:
--
--/code
--      abortErr( {"The tape drive is not responding.", /w32MsgNum + 17} )
--
--      abortErr( "The file '%s' was not found.", /w32MsgNum + 23, {filename})
--
--      abortErr(  "Fatal error. Bummer, dude." )
--/endcode


global procedure abortErr( sequence pParams )
   fShowError(pParams, 2)
end procedure


-----------------------------------------------------------------------------
--/topic Utilities
--/proc warnErr( sequence errorparam )
--/desc Display an error message, with option to abort.
-- /i errorparam is a one, two or three element sequence. /n
-- The first element is a text message. This is mandatory. /n
-- The second element is an error code (integer). The default is zero. /n
-- The third elements is a list of values for the "sprint" pattern in the
-- first element (a sequence).
--
-- This routine will display a dialog window with three buttons. /n
-- /i"[ YES ]" : If selected, the program will continue. /n
-- /i"[ NO  ]" : if pressed, the program will continue, but future warning about the /b same
-- error will be ignored. /n
-- /i"[CANCEL]": if pressed, will stop the program running immediately.
--
-- If any user-defined clean up routine have been attached, they are invoked prior to
-- win32lib's own cleanup routine, if the application is aborted. See /attachCleanUp()
-- for details.
-- Example:
--
--/code
--      warnErr( "Bad data. Abort program?" )
--      warnErr( "Good morning %s, this message will self-destruct in %d seconds",
--                  0, {"Mr Phelps", 5})
--/endcode

global procedure warnErr( sequence pParams )

   if vShowWarnings != 0
   then
      fShowError(pParams, vShowWarnings)
   end if

end procedure

-----------------------------------------------------------------------------
--/topic Utilities
--/proc setWarning( integer flag )
--/desc Shows or Hides warning messages from user.
-- Since 'warning' messages (triggered by /warnErr) are only warnings and
-- not fatal, it may be advantageous to suppress them in an application.
--
-- The /i flag maybe one of ... /n
--/li 0 To hide warning messages
--/li 1 To show warning messages, giving the option to continue or quit.
--/li 2 To turn all warning messages into fatal errors instead.
--
-- By default, the setting is /b 1
-- Example:
--
--/code                   
--      -- suppress warning messages
--      setWarning( 0 )
--/endcode

global procedure setWarning( integer flag )
sequence
     lRealFlag

   lRealFlag = {1,0,1,2}
   -- show or suppress warnErr messages
   vShowWarnings = lRealFlag[ find(flag, {0,1,2}) + 1 ]

end procedure

-----------------------------------------------------------------------------
--/topic Utilities
--/proc attachCleanUp( routine_id ) or ( {routine_id, {data...}} )
--/desc Establishes a user defined clean up routine which is invoked just prior to win32lib aborting.
--/ret The number of clean up routines attached.
-- Allows the application to clean up when win32lib detects an abort situation. It is possible to
-- attach multiple clean up routines. If this is done, they are invoked by win32lib in 
-- order of most-recently-attached to first-attached.
--
-- The clean up routine, when invoked by win32lib, is passed four parameters...
--/li /b ErrorCode (integer) The code number for the error that is causing win32lib to abort.
--/li /b ErrorText (sequence) The text displayed to the user.
--/li /b ControlId (integer) The id of the current control (0 => no control is current).
--/li /b LastCleanup (integer) A flag which is 1 if this is the last user defined cleanup
-- routine to be invoked, and 0 if there are others still to be invoked.
--
-- The clean up routine must return an integer /i flag. If the /i flag is -1, then no further attached
-- clean up routines will be invoked before win32lib aborts, otherwise any other routines will
-- be invoked.
--
-- Example:
--
--/code
--   function AppCleanUp(integer ErrCode, sequence ErrText, integer LastCleanUp, object UserData)
--      . . .
--      return 0 -- Continue with other clean up routines.
--   end function
--   
--   -- Link in my clean-up routine
--   cnt = attachCleanUp( routine_id("AppCleanUp" ))
--/endcode

global function attachCleanUp( object pId )
   integer lId
   sequence lData

    if sequence(pId) then
        lId = pId[1]
        lData = pId[2]
    else
        lId = pId
        lData = {}
    end if
    
    if lId >= 0
    then
        if atom(vUserCleanUp)
        then -- this is the first defined routine
            vUserCleanUp = {lId}
            vUserCleanUpData = {lData}
        else -- not the first defined routine
            vUserCleanUp = prepend(vUserCleanUp, lId)
            vUserCleanUpData = prepend(vUserCleanUpData, lData)
        end if

        return length(vUserCleanUp)
    else
      return lId
    end if
   
end function

-----------------------------------------------------------------------------
--/topic Utilities
--/proc detachCleanUp( routine_id )
--/desc Removes a user defined clean up routine from the list of attached ones.
--/ret If unsuccessful this returns -1, else the number of clean up routines still attached.
--
-- Example:
--
--/code
--   integer cnt, CU_id
--   . . . 
--   function AppCleanUp(integer ErrCode, sequence ErrText, integer ControlId, integer LastCleanUp)
--      . . .
--      return 0 -- Continue with other clean up routines.
--   end function
--   
--   -- Link in my clean-up routine
--   CU_id = routine_id("AppCleanUp" )
--   cnt = attachCleanUp( CU_id )
--   . . . 
--   cnt = detachCleanUp( CU_id ) 
--/endcode

global function detachCleanUp( integer pId )
integer lRc
   if atom(vUserCleanUp)
   then
      return -1
   end if
   
   lRc = find( pId, vUserCleanUp)
   
   if lRc > 0
   then
      if length(vUserCleanUp) = 1
      then
         vUserCleanUp = -1
         vUserCleanUpData = -1
         return 0
      else
         vUserCleanUp = vUserCleanUp[1 .. lRc - 1] & vUserCleanUp[lRc + 1 .. length(vUserCleanUp)]
         vUserCleanUpData = vUserCleanUpData[1 .. lRc - 1] & vUserCleanUpData[lRc + 1 .. length(vUserCleanUpData)]
         return length(vUserCleanUp)
      end if
   else
      return -1
   end if
   
end function 

--/topic Utilities
--/proc InitErrorSubSys( sequence pAppName, sequence pVersion )
--/desc Initializes the error message subsystem.
-- /b pAppName is the application's name /n
-- /b pVersion is the application's version

global function InitErrorSubSys(  sequence pAppName, sequence pVersion )
    vAppName = pAppName
    vVersion = pVersion
    return routine_id("fShowError")
end function
