--------------------NOTICE-------------------------------*
-- Software ID: w32utils.ew
-- Version:     0.54 "Bleeding Edge" 07/Oct/200
-- Copyright:   (c) 2000 David Cuny and friends
--              All rights reserved.
-- Licence:            
-- This software is provided 'as-is', without any express or implied warranty.
-- In no event will the authors be held liable for any damages arising from
-- the use of this software.
--
-- Permission is granted to anyone to use this software for any purpose,
-- including commercial applications, and to alter it and redistribute it
-- freely, subject to the following restrictictions:
-- 1. The origin of this software must not be misrepresented; you must not
--    claim that you wrote the original software.
-- 2. If you use this software in a product, acknowedgement in the product's
--    documenation and binary are required.
-- 3. Altered source versions, and works substantially derived from the it,
--    must...
--   a) be plainly be marked as such,
--   b) not be misrepresented as the original software,
--   c) include this notice, unaltered.
--------------------End of NOTICE------------------------*
include misc.e

--/topic Utilities
--/info
--Miscellaneous "utility" type routihes.
--These are miscellaneous utility functions and procedures that can be used
--with many Euphoria projects.

--/topic Utilities
--/const True 
--/desc Defines the Euphoria TRUE value
--/const False
--/desc Defines the Euphoria FALSE value

global constant
    False   = (1 = 0),
    True    = (1 = 1)

--/topic Utilities
--/func findKey( key, list )
--/desc Find key in list.
--/ret Index of sublist with matching key.
--
-- Example:
--
--/code 
--      -- find a value from a list
--      integer at
--      at = findKey( name, nameList )
--/endcode

global function findKey( object key, sequence list )
    for i = 1 to length( list ) do
        if equal( list[i][1], key ) then
            return i
        end if
    end for
    return 0

end function


----------------------------------------------------------------------------
--/topic Utilities
--/func removeIndex( integer index, sequence list )
--/desc Remove the element at position /i index from the sequence /i list
--/ret The /i list without the element.
--
-- Example:
--
--/code
--      -- Remove the 4th element from the Name List
--      nameList = removeIndex( 4, nameList )
--/endcode

global function removeIndex( integer index, sequence list )
    -- Method by Michael Palffy
    -- remove item from list at index postition
    -- ex:  removeIndex( 2, "abcd" )
    --      --> "acd"
    integer len, a, z

    len = length(list)

    -- Test for the smallest # of elements to move.
    if index < 1 or index > len then
        return list
    elsif index = 1 then
        a = 2
        z = len
    elsif index = len then
        a = 1
        z = len - 1
    elsif index < (len - index)  then
        list[2 .. index] = list[1 .. index - 1] -- move front portion up
        a = 2
        z = len
    else
        list[index .. len-1] = list[index + 1 .. len] -- move back portion down
        a = 1
        z = len - 1
    end if

    return list[a .. z]
end function

--/topic Utilities
--/func insertIndex( integer index, sequence list, object Data )
--/desc Inserts /i Data at position /i index in the sequence /i list
--/ret The /i list with the new element.
--If /i index is zero or less, the /i Data is added to the front of the /i list
-- and if /i index is passed the end of the list, /i Data is added to the
-- end of the /i list.
--
-- Example:
--
--/code
--      -- Insert a new name into position 4 of the list
--      nameList = insertIndex( 4, nameList, "Michael Palffy" )
--/endcode

----------------------------------------------------------------------------
global function insertIndex(integer index, sequence list, object pData)
    -- Method by Michael Palffy
    integer len

    len = length(list) + 1

    if index < 1 then
        return prepend(list, pData)
    elsif index >= len then
        return append(list, pData)
    -- Test for the smallest # of elements to move.
    elsif index < (len - index) then
        list = 0 & list  -- reserve space at the front
        list[1 .. index-1] = list[2 .. index] -- move front portion down
    else
        list &= 0    -- reserve some space at the end
        list[index+1 .. len] = list[index .. len-1] -- move back portion up
    end if
    list[index] = pData -- assign the new value

    return list
end function

--/topic Utilities
--/func iff (atom test, object ifTrue, object ifFalse)
--/desc Used to embed an 'if' test inside an expression.
--/ret If /i test is /b true then /i ifTrue is returned otherwise /i ifFalse is returned.
--
--Example
--/code
--         msg = sprintf("%s: %s", {
--                      iff(ErrType = 'E', "Fatal error", "Warning"),
--                      errortext } )
--/endcode

----------------------------------------------------------------------------
global function iff( atom test, object ifTrue, object ifFalse )
    -- returns ifTrue if flag is true, else returns ifFalse
    -- ex:  ? iff( 1, "this", "that" )
    --      --> "this"
    if test then
        return ifTrue
    else
        return ifFalse
    end if

end function

--/topic Utilities
--/func removeItem( object item, sequence list)
--/desc Removes /i item from the /i list, if it is in the list.
--/ret The /i list without the item.
--
--Example
--/code
--     -- Remove the name 'fred' from the list.
--     nameList = removeItem("fred", nameList)
--/endcode
----------------------------------------------------------------------------
global function removeItem( object item, sequence list )
    -- Removes the item from the list if item is in the list.
    return removeIndex(find(item,  list ), list)
end function


--/topic Utilities
--/func removeKey( object item, sequence list)
--/desc Removes /i keyed item from the /i list, if it is in the list.
--/ret The /i list without the item.
--The difference between this and /removeItem is that in this function,
-- /i list is a list of sequences and /i item is found if it matches the
-- first element in the sequences. /n
--Example
--/code
--     -- Remove the tracked resource.
--     ResourceList = removeKey(hWnd, ResourceList)
--     -- Add new tracked item.
--     ResourceList = insertIndex(0, ResourceList, {hWnd, 0, TagData})
--
--/endcode
----------------------------------------------------------------------------
global function removeKey( object item, sequence list )
    -- Removes the keyed item from the list if item is in the list.
    return removeIndex(findKey(item,  list ), list)
end function


-----------------------------------------------------------------------------
global function codeToBytes( sequence bits )
    -- Convert a sequence of text into bytes
    -- This is a support routine for createMonochromeBitmap
    -- Ex:  "1,1,1,1,1,1,1,1,0,0"
    --      --> { #FF, #00 }    
    
    integer byte, extra
    sequence slice, bytes
    
    -- add extra bits: must be multiple of 16
    extra = remainder( length(bits), 16 )
    if extra then
        bits = bits & repeat( 1, 16-extra )        
    end if
    
    -- convert bits to bytes
    bytes = {}       
    
    for i = 1 to length( bits ) by 8 do  
        -- get an 8 bit slice
        slice = bits[i..i+7]
        -- reverse it for conversion
        slice = reverse( slice )
        -- convert bits to a byte
        byte = bits_to_int( slice )
        -- add to list
        bytes = append( bytes, byte )
    end for   
    return bytes
end function

--/topic Utilities
--/func unique(sequence s)
--/desc Ensures that each element of /i s only occurs once in /i s
--/ret SEQUENCE: /i s but with duplicate elements removed.
--
--Example:
--/code
--        sequence x
--        x = unique("the quick brown fox jumped over the lazy dog")
--        -- x should be "the quickbrownfxjumpdvlazyg"
--/endcode
global function unique(sequence s)
    sequence lResult
    integer lOld, lNew, lScan

    lResult = {}

    for i = 1 to length(s) do
        if find(s[i], lResult) = 0 then
            lResult = append(s[i], lResult)
        end if
    end for

    return lResult
    
end function
